@@ -356,6 +356,22 @@ impl Address {
 
         Ok(pages_to_remove)
     }
+    pub fn recover_segment_operations(&self, seg_ops: &[SegmentOperation]) -> PERes<()> {
+        let mut segments = self.segments.write()?;
+        for seg_op in seg_ops {
+            if let SegmentOperation::Drop(ref op) = *seg_op {
+                segments.drop_segment(&op.name);
+            }
+        }
+
+        for seg_op in seg_ops {
+            if let SegmentOperation::Create(ref op) = *seg_op {
+                segments.recover_create_segment(op.segment_id, &op.name, op.first_page);
+            }
+        }
+        segments.flush_segments(&self.allocator)?;
+        Ok(())
+    }
 
     pub fn collect_segment_pages(&self, segment: SegmentId) -> PERes<Vec<u64>> {
         let segments = self.segments.read()?;
