@@ -6,6 +6,7 @@ use crate::{
     io::{read_u64, write_u64, InfallibleRead, InfallibleReadFormat},
     persy::exp_from_content_size,
     segment::{SEGMENTS_ROOT_PAGE_VERSION, SEGMENTS_ROOT_PAGE_VERSION_0},
+    snapshots::SnapshotId,
 };
 use linked_hash_map::LinkedHashMap;
 use std::{
@@ -197,6 +198,11 @@ struct RootMonitor {
     address_holder: RootPageHolder,
 }
 
+#[derive(Default)]
+struct ReleaseNextSync {
+    to_release: Vec<SnapshotId>,
+}
+
 // TODO: Manage defragmentation by merging/splitting pages in the free list
 pub struct Allocator {
     disc: Box<dyn Device>,
@@ -204,6 +210,7 @@ pub struct Allocator {
     cache: Mutex<Cache>,
     root_monitor: Mutex<RootMonitor>,
     flush_count: Mutex<FlushCount>,
+    release_next_sync: Mutex<ReleaseNextSync>,
     page: u64,
 }
 
@@ -237,6 +244,7 @@ impl Allocator {
             cache: Mutex::new(Cache::new(cache_size, cache_age_limit)),
             root_monitor: Mutex::new(root_monitor),
             flush_count: Mutex::new(flush_count),
+            release_next_sync: Default::default(),
             page,
         })
     }
@@ -276,6 +284,10 @@ impl Allocator {
         }
     }
 
+    pub fn to_release_next_sync(&self, snapshot_id: SnapshotId) {
+        self.release_next_sync.lock().unwrap().to_release.push(snapshot_id);
+    }
+
     pub fn load_page(&self, page: u64) -> PERes<ReadPage> {
         let load = self.read_page_int(page)?;
         debug_assert!(!load.is_free()?, "page {} should not be marked as free", page);
@@ -396,6 +408,8 @@ impl Allocator {
         if let Ok(p) = self.disc.load_page(page) {
             if !p.is_free()? {
                 self.free(page)?;
+            } else if p.is_free_list_head()? {
+                self.free_list.lock()?.set(p.get_size_exp(), p.get_index());
             }
         }
         Ok(())
@@ -580,7 +594,7 @@ impl Allocator {
         &*self.disc
     }
 
-    pub fn disc_sync(&self) -> PERes<()> {
+    pub fn disc_sync(&self) -> PERes<Vec<SnapshotId>> {
         let free_list_data;
         let journal_data;
         let address_data;
@@ -604,7 +618,9 @@ impl Allocator {
             }
             self.disc.sync()?;
         }
-        Ok(())
+        let result = std::mem::take(&mut self.release_next_sync.lock()?.to_release);
+
+        Ok(result)
     }
 
     pub fn release(self) -> Box<dyn Device> {
@@ -619,14 +635,17 @@ impl UpdateList for FreeList {
         Ok(old)
     }
 
-    fn remove(&mut self, size: u8, page: u64, next: u64, check: bool) -> PERes<()> {
-        let old = self.set(size, next);
+    fn remove(&mut self, size: u8, _page: u64, next: u64, _check: bool) -> PERes<()> {
+        let _old = self.set(size, next);
+        /*
+        This can fail in some specific case with the current free list management, disabled for now
         debug_assert!(
             old == page || old == 0 || !check,
             "trimmed page not in top list expected:{} current:{} ",
             page,
             old
         );
+        */
         Ok(())
     }
 }
