@@ -1,5 +1,8 @@
 use crate::{
-    device::{free_flag_set, is_free, Page, ReadPage, SizeTool, UpdateList, FLAGS_OFFSET, NEXT_OFFSET, PREV_OFFSET},
+    device::{
+        free_flag_set, free_list_head_flag_set, is_free, Page, ReadPage, SizeTool, UpdateList, FLAGS_OFFSET,
+        NEXT_OFFSET, PREV_OFFSET,
+    },
     error::PERes,
     io::{read_u64, write_u64, ReadFormat, WriteFormat},
 };
@@ -62,11 +65,17 @@ pub(crate) fn mark_allocated<T: Write + Read + Seek>(fl: &mut T, page: u64) -> P
     // Free pages are a linked list reading the next page to return
     let next = fl.read_u64()?;
     moderator = free_flag_set(moderator, false);
+    moderator = free_list_head_flag_set(moderator, false);
     fl.seek(SeekFrom::Start(page + 1))?;
     fl.write_u8(moderator)?;
     fl.write_u64(0)?; // next
     fl.write_u64(0)?; // previous
     if next != 0 {
+        fl.seek(SeekFrom::Start(next + FLAGS_OFFSET as u64))?;
+        let mut mitigator = fl.read_u8()?;
+        mitigator = free_list_head_flag_set(mitigator, true);
+        fl.seek(SeekFrom::Start(next + FLAGS_OFFSET as u64))?;
+        fl.write_u8(mitigator)?;
         fl.seek(SeekFrom::Start(next + PREV_OFFSET as u64))?;
         fl.write_u64(0)?;
     }
@@ -90,6 +99,7 @@ pub(crate) fn trim_or_free_page<T: Write + Read + Seek + SizeTool>(
         let next = update_list.update(exp, page)?;
         fl.seek(SeekFrom::Start(page + 1))?;
         mitigator = free_flag_set(mitigator, true);
+        mitigator = free_list_head_flag_set(mitigator, true);
         let mut data: [u8; 9] = [0; 9];
         data[0] = mitigator;
         write_u64(&mut data[1..9], next);
@@ -142,6 +152,13 @@ pub(crate) fn check_and_trim<T: Write + Read + Seek + SizeTool>(
             fl.write_all(&data)?;
         }
         if next != 0 {
+            if prev == 0 {
+                fl.seek(SeekFrom::Start(next + FLAGS_OFFSET as u64))?;
+                let mut mitigator = fl.read_u8()?;
+                mitigator = free_list_head_flag_set(mitigator, true);
+                fl.seek(SeekFrom::Start(next + FLAGS_OFFSET as u64))?;
+                fl.write_u8(mitigator)?;
+            }
             let mut data = [0u8; 8];
             write_u64(&mut data, prev);
             fl.seek(SeekFrom::Start(next + PREV_OFFSET as u64))?;
