@@ -10,6 +10,7 @@ use crate::{
         Cleanup, Commit, CreateSegment, DeleteRecord, DropSegment, FreedPage, InsertRecord, Metadata, NewSegmentPage,
         PrepareCommit, ReadRecord, Rollback, Start, UpdateRecord,
     },
+    snapshots::Snapshots,
     transaction_impl::TransactionImpl,
     RecoverStatus,
 };
@@ -78,11 +79,15 @@ impl StartList {
             false
         }
     }
-}
 
-struct JournalPagesToFree {
-    free_tx_id: JournalId,
-    pages: Vec<u64>,
+    fn is_page_in_start_list(&self, page: u64) -> bool {
+        for (k, _v) in &self.transactions {
+            if k.page == page {
+                return true;
+            }
+        }
+        false
+    }
 }
 
 struct JournalShared {
@@ -93,7 +98,6 @@ struct JournalShared {
     starts: StartList,
     current: Page,
     to_clear: Vec<JournalId>,
-    to_free: Option<JournalPagesToFree>,
 }
 
 impl JournalShared {
@@ -118,7 +122,6 @@ impl JournalShared {
             starts: StartList::new(),
             current,
             to_clear: Vec::new(),
-            to_free: None,
         })
     }
 
@@ -179,7 +182,7 @@ impl JournalShared {
         Ok(id)
     }
 
-    pub fn clear_in_queue(&mut self, allocator: &Allocator) -> PERes<()> {
+    pub fn clear_in_queue(&mut self, allocator: &Allocator, snapshots: &Snapshots) -> PERes<()> {
         let mut pages_to_free = Vec::new();
         let ids = self.to_clear.clone();
         self.to_clear.clear();
@@ -196,9 +199,12 @@ impl JournalShared {
                         self.current.seek(pos as u32);
                         prev
                     } else {
-                        let mut cur = allocator.load_page(free_cursor)?;
-                        cur.seek(JOURNAL_PAGE_PREV_OFFSET);
-                        cur.read_u64()
+                        if let Some(mut cur) = allocator.load_page_not_free(free_cursor)? {
+                            cur.seek(JOURNAL_PAGE_PREV_OFFSET);
+                            cur.read_u64()
+                        } else {
+                            break;
+                        }
                     };
                     if free_cursor != id.page {
                         pages_to_free.push(free_cursor)
@@ -218,33 +224,30 @@ impl JournalShared {
             let root = allocator.write_page(self.root)?;
             allocator.write_journal_root(root, &mut buffer, JOURNAL_ROOT_VERSION)?;
         }
-        self.free_pages_tx(pages_to_free, allocator)?;
+        self.free_pages_tx(pages_to_free, allocator, snapshots)?;
         Ok(())
     }
 
-    pub fn free_pages_tx(&mut self, pages_to_free: Vec<u64>, allocator: &Allocator) -> PERes<()> {
-        //tx
-        let to_free_list = self.to_free.take();
-        if let Some(to_free) = to_free_list {
-            for page in to_free.pages {
-                allocator.free(page)?;
-            }
-            // Enqueue for clear the free pages journal pages tx
-            self.finished_to_clean(&[to_free.free_tx_id], allocator)?;
-            allocator.flush_free_list()?;
-        }
+    pub fn free_pages_tx(
+        &mut self,
+        pages_to_free: Vec<u64>,
+        allocator: &Allocator,
+        snapshots: &Snapshots,
+    ) -> PERes<()> {
         if !pages_to_free.is_empty() {
             // Log the pages removed from the journal for free
             let id = self.start(allocator)?;
+            let mut pages_log = Vec::new();
+
             for page in &pages_to_free {
-                self.log_bytes(&Journal::prepare_buffer(&FreedPage::new(*page), &id)?, allocator, false)?;
+                let free_page = FreedPage::new(*page);
+                self.log_bytes(&Journal::prepare_buffer(&free_page, &id)?, allocator, false)?;
+                pages_log.push(free_page)
             }
             self.log_bytes(&Journal::prepare_buffer(&PrepareCommit::new(), &id)?, allocator, false)?;
             self.log_bytes(&Journal::prepare_buffer(&Commit::new(), &id)?, allocator, true)?;
-            self.to_free = Some(JournalPagesToFree {
-                free_tx_id: id,
-                pages: pages_to_free,
-            });
+            let snapshot_id = snapshots.snapshot(Vec::new(), pages_log, id)?;
+            allocator.to_release_next_sync(snapshot_id);
         }
         Ok(())
     }
@@ -384,8 +387,8 @@ impl Journal {
         Ok(())
     }
 
-    pub fn clear_in_queue(&self) -> PERes<()> {
-        self.journal.lock()?.clear_in_queue(&self.allocator)
+    pub fn clear_in_queue(&self, snapshots: &Snapshots) -> PERes<()> {
+        self.journal.lock()?.clear_in_queue(&self.allocator, snapshots)
     }
 
     pub fn finished_to_clean(&self, ids: &[JournalId]) -> PERes<()> {
@@ -418,6 +421,10 @@ impl Journal {
         jr.log_bytes(&buffer, &self.allocator, flush)
     }
 
+    pub fn is_page_in_start_list(&self, page: u64) -> bool {
+        self.journal.lock().unwrap().starts.is_page_in_start_list(page)
+    }
+
     pub(crate) fn recover<T>(&self, found: T) -> PERes<()>
     where
         T: FnMut(&dyn JournalEntry, &JournalId),
