@@ -144,14 +144,22 @@ fn create_background_ops(
     snapshots: Arc<Snapshots>,
 ) -> PERes<BackgroundOps<SnapshotId>> {
     let all_sync = allocator.clone();
+    let snap_sync = snapshots.clone();
+    let journal_sync = journal.clone();
     BackgroundOps::new(
-        move || all_sync.disc_sync(),
+        move || {
+            let to_release = all_sync.disc_sync()?;
+            for snap in to_release {
+                release_snapshot(snap, &snap_sync, &all_sync, &journal_sync)?;
+            }
+            Ok(())
+        },
         move |all: &[SnapshotId]| {
             for snap in all {
                 release_snapshot(*snap, &snapshots, &allocator, &journal)?;
             }
             if !all.is_empty() {
-                journal.clear_in_queue()?;
+                journal.clear_in_queue(&snapshots)?;
             }
             Ok(())
         },
@@ -197,7 +205,8 @@ impl PersyImpl {
             root.write_all(PERSY_TAG_BYTES);
             allocator.flush_page(root)?;
         }
-        allocator.disc_sync()?;
+        let res = allocator.disc_sync()?;
+        debug_assert!(res.is_empty());
         Ok(allocator.release())
     }
 
@@ -311,8 +320,11 @@ impl PersyImpl {
         }
         self.address.recompute_last_pages()?;
         self.address.flush_segments()?;
-        allocator.disc_sync()?;
-        self.journal.clear_in_queue()?;
+        let to_release = allocator.disc_sync()?;
+        for snap in to_release {
+            release_snapshot(snap, &self.snapshots(), &self.allocator(), &self.journal())?;
+        }
+        self.journal.clear_in_queue(&self.snapshots())?;
         allocator.trim_free_at_end()?;
         Ok(())
     }
@@ -1068,8 +1080,11 @@ impl PersyImpl {
         match sync_mode {
             SyncMode::Sync => {
                 let allocator = self.allocator();
-                allocator.disc_sync()?;
-                self.journal().clear_in_queue()?;
+                let to_release = allocator.disc_sync()?;
+                for snap in to_release {
+                    release_snapshot(snap, &self.snapshots(), &self.allocator(), &self.journal())?;
+                }
+                self.journal().clear_in_queue(&self.snapshots())?;
             }
             SyncMode::BackgroundSync => {
                 self.snapshots.acquire(snapshot_id)?;
@@ -1084,8 +1099,11 @@ impl PersyImpl {
         match sync_mode {
             SyncMode::Sync => {
                 let allocator = self.allocator();
-                allocator.disc_sync()?;
-                self.journal().clear_in_queue()?;
+                let to_release = allocator.disc_sync()?;
+                for snap in to_release {
+                    release_snapshot(snap, &self.snapshots(), &self.allocator(), &self.journal())?;
+                }
+                self.journal().clear_in_queue(&self.snapshots())?;
             }
             SyncMode::BackgroundSync => unreachable!(),
         }
