@@ -451,6 +451,15 @@ impl Segments {
         }
     }
 
+    pub fn recover_create_segment(&mut self, segment: SegmentId, name: &str, first_page: u64) {
+        if !self.segments.contains_key(&segment) {
+            let s = Segment::new(first_page, first_page, SEGMENT_DATA_OFFSET, segment, name);
+            self.temp_segments_id.remove(&s.name);
+            self.segments_id.insert(s.name.clone(), s.segment_id);
+            self.segments.insert(s.segment_id, s);
+        }
+    }
+
     pub fn drop_segment(&mut self, segment: &str) {
         if let Some(seg) = self.segments_id.remove(segment) {
             self.segments.remove(&seg);
@@ -501,6 +510,8 @@ impl Segments {
                 }
                 segment.last_page = n_page;
             }
+            let mut page = allocator.load_page(segment.last_page)?;
+            segment.last_pos = page.segment_first_available_pos();
         }
         Ok(())
     }
