@@ -174,7 +174,7 @@ impl InternalSnapshots {
                                 .binary_search_by(|n| search(n.snapshot_id, snapshot_id, snapshot_sequence))
                             {
                                 Ok(index) => {
-                                    v.get_mut().truncate(index);
+                                    v.get_mut().drain(..=index);
                                     if v.get().is_empty() {
                                         v.remove();
                                     }
