@@ -766,9 +766,14 @@ impl TransactionImpl {
 
     pub fn recover_cleanup(&self, persy_impl: &PersyImpl) -> PERes<()> {
         let allocator = persy_impl.allocator();
+        let address = persy_impl.address();
+        let journal = persy_impl.journal();
+        address.recover_segment_operations(&self.segments_operations)?;
         if let Some(ref up_free) = self.freed_pages {
             for to_free in up_free {
-                allocator.recover_free(to_free.page)?;
+                if !journal.is_page_in_start_list(to_free.page) {
+                    allocator.recover_free(to_free.page)?;
+                }
             }
         }
         persy_impl.allocator().disc_sync()?;
@@ -850,14 +855,12 @@ impl TransactionImpl {
     pub fn commit(&mut self, persy_impl: &PersyImpl, prepared: PreparedState) -> Result<(), GenericError> {
         let allocator = persy_impl.allocator();
         let journal = persy_impl.journal();
-        let snapshots = persy_impl.snapshots();
 
         let address_to_free = self.internal_commit(persy_impl, false, &prepared)?;
         journal.end(&Commit::new(), &self.id)?;
         self.free_address_structures(address_to_free, persy_impl)?;
-
         if let Some(snapshot_id) = prepared.snapshot_id {
-            release_snapshot(snapshot_id, snapshots, allocator, journal)?;
+            allocator.to_release_next_sync(snapshot_id);
         }
         Ok(())
     }
